/*
	Copyright Berend "Kirk" Wouda 2004

	This file is part of WhatDoYouSee'd.

	WhatDoYouSee'd is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	WhatDoYouSee'd is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with WhatDoYouSee'd; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// The package, naturally.
package WDYSDecrypt;

// Import IOException.
import java.io.IOException;

/**
 * This class controls the decryption process. It makes sure the file is read in and
 * fed to the decrypting object, and makes sure the decrypted file is written
 * afterwards.
 *
 * @author Berend "Kirk" Wouda
 * @version 1
 * @see WDYSDecrypt.WDYSDecrypter
 * @see WDYSDecrypt.WDYSEncryptedFileReader
 * @see WDYSDecrypt.WDYSDecryptedFileWriter
 */
public class WDYSDecryptControl {
	/**
	 * Constructs a new decryption control object.
	 */
	public WDYSDecryptControl() {
		// Yay.
	}

	/**
	 * Decrypts the given file.
	 *
	 * @param filename1 The name of the file to be read from (and decrypted).
	 * @param filename2 The name of the file to be written to.
	 * @throws IOException When IO stuff goes wrong.
	 * @throws WDYSDecryptException When decrypt stuff goes wrong.
	 */
	public void decrypt(String filename1, String filename2) throws IOException, WDYSDecryptException {
		// Construct a file reader.
		WDYSEncryptedFileReader reader = new WDYSEncryptedFileReader(filename1);

		// Construct a file writer.
		WDYSDecryptedFileWriter writer = new WDYSDecryptedFileWriter(filename2, reader.getDecryptedDataLength());

		// Construct a decrypter.
		WDYSDecrypter decrypter = new WDYSDecrypter();

		// Continually ask the filereader for 2 times 4 bytes, give them to the
		// decrypter to decrypt it, and write the results to the new file, until
		// we run out of bytes.
		for(int index = 0; index < reader.getEncryptedDataLength(); index += 8) {
			// Read 8 bytes and feed them to the decrypter.
			writer.write8DecryptedDataBytes(decrypter.decryptQuad(reader.read4EncryptedDataBytes(), reader.read4EncryptedDataBytes()));
		}

		// Close the reader.
		reader.close();

		// Close the writer.
		writer.close();
	}
}