/*
	Copyright Berend "Kirk" Wouda 2004

	This file is part of WhatDoYouSee'd.

	WhatDoYouSee'd is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	WhatDoYouSee'd is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with WhatDoYouSee'd; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// The package, d'oh.
package WDYSEncrypt;

// Import the IO classes.
import java.io.*;

/**
 * This class reads decrypted WDYS files 4 bytes at a time.
 *
 * @author Berend "Kirk" Wouda
 * @version 1
 */
public class WDYSDecryptedFileReader {
	/**
	 * Constructs a new file reader around the given file.
	 *
	 * @param filename The name of the decrypted file.
	 * @throws IOException When IO stuff goes wrong.
	 */
	public WDYSDecryptedFileReader(String filename) throws IOException {
		// Construct the File object of the file indicated by filename. Yeah.
		decryptedfile = new File(filename);

		// Construct a new BufferedInputStream around this baby.
		// Well... Put in a FileInputStream just to be certain ;^)
		reader = new BufferedInputStream(new FileInputStream(decryptedfile));
	}


	/**
	 * Closes the file.
	 *
	 * @throws IOException When IO stuff goes wrong.
	 */
	public void close() throws IOException {
		// Yeah baby!
		reader.close();
	}

	/**
	 * Returns the length of the decrypted file.
	 *
	 * @return Said length.
	 */
	public long getDecryptedDataLength() {
		// The length is the length of the file, naturally.
		return decryptedfile.length();
	}

	/**
	 * Reads and returns the next 4 bytes from the decrypted data file.
	 *
	 * @return What I just said above, jeez.
	 * @throws IOException When IO stuff goes wrong.
	 * @throws WDYSEncryptException When the 4 bytes could not be supplied.
	 */
	public byte[] read4DecryptedDataBytes() throws IOException, WDYSEncryptException {
		// A byte array.
		byte[] quad = new byte[4];

		// Read.
		if(reader.read(quad, 0, 4) != 4) throw new WDYSEncryptException("EOF");

		// Return.
		return quad;
	}

	/**
	 * Reads and returns the next given amount of bytes from the decrypted data file.
	 *
	 * @param amount The amount of bytes to be read.
	 * @return What I just said above, jeez.
	 * @throws IOException When IO stuff goes wrong.
	 * @throws WDYSEncryptException When the given amount of bytes could not be
	 * supplied.
	 */
	public byte[] readDecryptedDataBytes(int amount) throws IOException, WDYSEncryptException {
		// A byte array.
		byte[] array = new byte[amount];

		// Read.
		if(reader.read(array, 0, amount) != amount) throw new WDYSEncryptException("Could not fully read the amount of bytes required. Either a file read error, or a bad program call.");

		// Return.
		return array;
	}


	/**
	 * The file.
	 */
	protected File decryptedfile;

	/**
	 * The reader. Well it's a stream actually.
	 */
	protected BufferedInputStream reader;
}