/*
	Copyright Berend "Kirk" Wouda 2004

	This file is part of WhatDoYouSee'd.

	WhatDoYouSee'd is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	WhatDoYouSee'd is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with WhatDoYouSee'd; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// The package, naturally.
package WDYSEncrypt;

// Import IOException.
import java.io.IOException;

/**
 * This class controls the encryption process. It makes sure the file is read in and
 * fed to the encrypting object, and makes sure the encrypted file is written
 * afterwards.
 *
 * @author Berend "Kirk" Wouda
 * @version 1
 * @see WDYSEncrypt.WDYSEncrypter
 * @see WDYSEncrypt.WDYSDecryptedFileReader
 * @see WDYSEncrypt.WDYSEncryptedFileWriter
 */
public class WDYSEncryptControl {
	/**
	 * Constructs a new decryption control object.
	 */
	public WDYSEncryptControl() {
		// Yay.
	}

	/**
	 * Encrypts the given file.
	 *
	 * @param filename1 The name of the file to be read from (and encrypted).
	 * @param filename2 The name of the file to be written to.
	 * @throws IOException When IO stuff goes wrong.
	 * @throws WDYSEncryptException When encrypt stuff goes wrong.
	 */
	public void encrypt(String filename1, String filename2) throws IOException, WDYSEncryptException {
		// Construct a file reader.
		WDYSDecryptedFileReader reader = new WDYSDecryptedFileReader(filename1);

		// Construct a file writer.
		WDYSEncryptedFileWriter writer = new WDYSEncryptedFileWriter(filename2, reader.getDecryptedDataLength());

		// Construct a encrypter.
		WDYSEncrypter encrypter = new WDYSEncrypter();

		// The amounf of bytes already written.
		long written;

		// Continually ask the filereader for 2 times 4 bytes, give them to the
		// encrypter to encrypt it, and write the results to the new file, until
		// we run out of bytes.
		for(written = 0; written < reader.getDecryptedDataLength() - 8; written += 8) {
			// Read 8 bytes and feed them to the encrypter.
			writer.write8EncryptedDataBytes(encrypter.encryptQuad(reader.read4DecryptedDataBytes(), reader.read4DecryptedDataBytes()));
		}

		// There are less than 8 bytes left. Calculate the exact amount.
		int left = (int) (reader.getDecryptedDataLength() - written);

		// Read the rest of the bytes.
		byte[] rest = reader.readDecryptedDataBytes(left);

		// Copy this array into a new array of length 8.
		byte[] padded = new byte[8];
		System.arraycopy(rest, 0, padded, 0, left);

		// Divide this array into 2 quads.
		byte[] rest1 = new byte[4];
		byte[] rest2 = new byte[4];
		System.arraycopy(padded, 0, rest1, 0, 4);
		System.arraycopy(padded, 4, rest2, 0, 4);

		// Encrypt the 2 quads and write them to the file.
		writer.write8EncryptedDataBytes(encrypter.encryptQuad(rest1, rest2));

		// Close the reader.
		reader.close();

		// Close the writer.
		writer.close();
	}
}