/*
	Copyright Berend "Kirk" Wouda 2004

	This file is part of WhatDoYouSee'd.

	WhatDoYouSee'd is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	WhatDoYouSee'd is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with WhatDoYouSee'd; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// The package, I see?
package gui;

// Import the AWT classes.
import java.awt.*;

// Import the control class.
import system.WDYSControl;

// Import the GridSpan classes.
import kirk.gui.layout.GridSpanLayout;
import kirk.gui.layout.GridSpanConstraints;

// Import the WindowCloser class.
import kirk.gui.closer.WindowCloser;

// Import the ErrorDialog class.
import kirk.gui.dialog.ErrorDialog;
;

/**
 * This class is the main window of WhatDoYouSee'd.
 *
 * @author Berend "Kirk" Wouda
 * @version 1
 */
public class WDYSFrame extends Frame {
	/**
	 * Constructs a new <code>WDSYFrame</code> and initialises it.
	 */
	public WDYSFrame(){
		// Super trooper la la la lala laaaa
		super("WhatDoYouSee'd v2.00");

		// Set the control class.
		control = new WDYSControl(this);

		// Make the window closable using a WindowCloser. Set it to exit the program
		// on closing.
		addWindowListener(new WindowCloser(true));

		// Set the size of this window to 400x200.
		setSize(400, 150);

		// Make the window unresizable.
		setResizable(false);

		// Set the location (in the middle of the screen).
		Dimension screensize = Toolkit.getDefaultToolkit().getScreenSize();
		setLocation((screensize.width - getWidth()) / 2, (screensize.height - getHeight())/ 2);

		// Set the layout to a 6x4 GridSpanLayout with 5 pixel gaps.
		setLayout(new GridSpanLayout(6, 4, 4, 2));

		// The constraints used for adding.
		GridSpanConstraints constraints;

		// Add the input label.
		constraints = new GridSpanConstraints();
		constraints.setX(0);
		constraints.setY(0);
		constraints.setWidth(3);
		constraints.setHeight(1);
		constraints.setHorizontalAnchor(0.0);
		constraints.setVerticalAnchor(1.0);
		Label inputlabel = new Label("Input File:", Label.LEFT);
		add(inputlabel, constraints);

		// Add the input browse button.
		constraints = new GridSpanConstraints();
		constraints.setX(3);
		constraints.setY(0);
		constraints.setWidth(1);
		constraints.setHeight(1);
		constraints.setVerticalAnchor(1.0);
		Button inputbrowsebutton = new Button("Browse...");
		inputbrowsebutton.setActionCommand("INPUTBROWSE");
		inputbrowsebutton.addActionListener(control);
		add(inputbrowsebutton, constraints);

		// Add the input field.
		constraints = new GridSpanConstraints();
		constraints.setX(0);
		constraints.setY(1);
		constraints.setWidth(4);
		constraints.setHeight(1);
		constraints.setHorizontalFill(1.0);
		constraints.setVerticalAnchor(0.0);
		inputfield = new TextField();
		add(inputfield, constraints);

		// Add the output label.
		constraints = new GridSpanConstraints();
		constraints.setX(0);
		constraints.setY(2);
		constraints.setWidth(3);
		constraints.setHeight(1);
		constraints.setHorizontalAnchor(0.0);
		constraints.setVerticalAnchor(1.0);
		Label outputlabel = new Label("Output File:", Label.LEFT);
		add(outputlabel, constraints);

		// Add the output browse button.
		constraints = new GridSpanConstraints();
		constraints.setX(3);
		constraints.setY(2);
		constraints.setWidth(1);
		constraints.setHeight(1);
		constraints.setVerticalAnchor(1.0);
		Button outputbrowsebutton = new Button("Browse...");
		outputbrowsebutton.setActionCommand("OUTPUTBROWSE");
		outputbrowsebutton.addActionListener(control);
		add(outputbrowsebutton, constraints);

		// Add the output field.
		constraints = new GridSpanConstraints();
		constraints.setX(0);
		constraints.setY(3);
		constraints.setWidth(4);
		constraints.setHeight(1);
		constraints.setHorizontalFill(1.0);
		constraints.setVerticalAnchor(0.0);
		outputfield = new TextField();
		add(outputfield, constraints);

		// Add the decrypt button.
		constraints = new GridSpanConstraints();
		constraints.setX(4);
		constraints.setY(0);
		constraints.setWidth(2);
		constraints.setHeight(2);
		constraints.setHorizontalFill(0.9);
		constraints.setVerticalFill(0.9);
		Button decryptbutton = new Button("Decrypt");
		decryptbutton.setActionCommand("DECRYPT");
		decryptbutton.addActionListener(control);
		add(decryptbutton, constraints);

		// Add the encrypt button.
		constraints = new GridSpanConstraints();
		constraints.setX(4);
		constraints.setY(2);
		constraints.setWidth(2);
		constraints.setHeight(2);
		constraints.setHorizontalFill(0.9);
		constraints.setVerticalFill(0.9);
		Button encryptbutton = new Button("Encrypt");
		encryptbutton.setActionCommand("ENCRYPT");
		encryptbutton.addActionListener(control);
		add(encryptbutton, constraints);

		// Make the window visible.
		setVisible(true);

		// Put the focus on the inputfield.
		inputfield.requestFocus();
	}


	/**
	 * Pops up a window with a message in it.
	 *
	 * @param message The message to be displayed.
	 */
	public void popup(String message) {
		// Create the popup.
		ErrorDialog popup = new ErrorDialog(this, message, message.length() * 7, 90);

		// Show the popup (this will also block this method until it is closed).
		popup.show();
	}

	/**
	 * Pops up a <code>FileDialog</code> and returns the selected file.
	 *
	 * @param title The title of the selection dialog.
	 * @param mode Whether it is about loading or saving.
	 * @return The selected filename.
	 */
	public String askForFileSelection(String title, int mode) {
		// Create a new FileDialog.
		FileDialog selection = new FileDialog(this, title, mode);

		// Show0r me.
		selection.show();

		// When we get here the user closed the selection dialog. Time to retrieve
		// and return that filename!
		return selection.getDirectory() + selection.getFile();
	}


	/**
	 * Returns the name of the input file the user selected/typed in.
	 *
	 * @return The name of the input file the user selected/typed in.
	 */
	public String getInputFilename() {
		// Return the input field contents.
		return inputfield.getText();
	}

	/**
	 * Sets the name of the input file.
	 *
	 * @param filename The name of the input file to be set.
	 */
	public void setInputFilename(String filename) {
		// Set the input field contents.
		inputfield.setText(filename);
	}


	/**
	 * Returns the name of the output file the user selected/typed in.
	 *
	 * @return The name of the output file the user selected/typed in.
	 */
	public String getOutputFilename() {
		// Return the output field contents.
		return outputfield.getText();
	}

	/**
	 * Sets the name of the output file.
	 *
	 * @param filename The name of the output file to be set.
	 */
	public void setOutputFilename(String filename) {
		// Set the output field contents.
		outputfield.setText(filename);
	}


	/**
	 * The input field.
	 */
	TextField inputfield;

	/**
	 * The output field.
	 */
	TextField outputfield;

	/**
	 * The control class.
	 */
	protected WDYSControl control;
}