/*
	Copyright Berend "Kirk" Wouda 2004

	This file is part of WhatDoYouSee'd.

	WhatDoYouSee'd is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	WhatDoYouSee'd is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with WhatDoYouSee'd; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// The package, one would expect.
package system;

// Import the GUI.
import gui.WDYSFrame;

// Import the event classes.
import java.awt.FileDialog;
import java.awt.event.*;

// Import the encyption and decryption classes.
import WDYSDecrypt.*;
import WDYSEncrypt.*;

// Import the IOException.
import java.io.IOException;

/**
 * This class processes all the user events.
 *
 * @author Berend "Kirk" Wouda
 * @version 1
 */
public class WDYSControl implements ActionListener {
	/**
	 * Constructs a new control class that processes events from the passed GUI.
	 *
	 * @param gui The GUI this control class works for.
	 */
	public WDYSControl(WDYSFrame gui) {
		// Set the GUI.
		this.gui = gui;
	}


	/**
	 * Processes an event caused by the GUI.
	 *
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent event) {
		// A button has been pressed.
		// Retrieve the action command from the event.
		String command = event.getActionCommand();

		// Find out which button was pressed.
		// The Decrypt button.
		     if(command.equals("DECRYPT"))  decrypt(gui.getInputFilename(), gui.getOutputFilename());
		// The Encrypt button.
		else if(command.equals("ENCRYPT"))  encrypt(gui.getInputFilename(), gui.getOutputFilename());
		// The Input Browse button.
		else if(command.equals("INPUTBROWSE"))  setInputFilename();
		// The Output Browse button.
		else if(command.equals("OUTPUTBROWSE"))  setOutputFilename();
		// Any other action.
		else {
			// This is bad.
		}
	}

	/**
	 * Decrypts the input file into the output file.
	 *
	 * @param in The input file.
	 * @param out The output file.
	 */
	protected void decrypt(String in, String out) {
		// Catch errors.
		try {
			// Construct a decrypt control object.
			WDYSDecryptControl control = new WDYSDecryptControl();

			// Ask it to decrypt the file.
			control.decrypt(in, out);

			// Please the user.
			gui.popup(in + " succesfully decrypted into " + out + ".");
		}
		catch(IOException e) {
			// Alert the user.
			gui.popup("An error occured while trying to read/write the file: " + e.getMessage());
		}
		catch(WDYSDecryptException e) {
			// Alert the user.
			gui.popup("An error occured while decrypting the file: " + e.getMessage());
		}
	}

	/**
	 * Encrypts the input file into the output file.
	 *
	 * @param in The input file.
	 * @param out The output file.
	 */
	protected void encrypt(String in, String out) {
		// Catch errors.
		try {
			// Construct an encrypt control object.
			WDYSEncryptControl control = new WDYSEncryptControl();

			// Ask it to encrypt the file.
			control.encrypt(in, out);

			// Please the user.
			gui.popup(in + " succesfully encrypted into " + out + ".");
		}
		catch(IOException e) {
			// Alert the user.
			gui.popup("An error occured while trying to read/write the file: " + e.getMessage());
		}
		catch(WDYSEncryptException e) {
			// Alert the user.
			gui.popup("An error occured while encrypting the file: " + e.getMessage());
		}
	}

	/**
	 * Asks the user to browse and select the input file.
	 */
	public void setInputFilename() {
		// Retrieve a file from the user.
		String filename = gui.askForFileSelection("Please select a file for input...", FileDialog.LOAD);

		// Set the input field to that filename.
		gui.setInputFilename(filename);
	}

	/**
	 * Asks the user to browse and select the input file.
	 */
	public void setOutputFilename() {
		// Retrieve a file from the user.
		String filename = gui.askForFileSelection("Please select a file for output...", FileDialog.SAVE);

		// Set the input field to that filename.
		gui.setOutputFilename(filename);
	}


	/**
	 * The GUI that this control class processes events for.
	 */
	protected WDYSFrame gui;
}